#!/bin/bash

### This script runs the model by varying a parameter value by a fixed amount from its default value
### and saves the output of interest (e.g. percentage change in DMS flux) into a text file.

echo "Choose the name of the text file for saving your results, followed by [ENTER]:"
read results

### INPUT starts ###
filename='fabm.nml'
pfac=(-0.1 0.1)
pname=(qi k_dmspdi k_dmsi k_phoi yieldi k_lyi f_exi k_exi)
pvalue=(9.5 3 0.5 0.1 0.05 0.05 0.05 0.02)
### INPUT ends ###

echo "Running the default simulation and saving its output (resolute_def.nc)..."
for (( i=0; i<${#pname[@]}; i++ ));
do
	vi $filename -c ":%s/\("${pname[i]}" \{1,}= \).*\(,\)/\1"${pvalue[i]}"\2/g" -c ":wq"
done
nice -n 19 gotm &> log.resolute
python sens_def.py
mv resolute.nc resolute_def.nc
required_file_size=$(wc -c <"resolute_def.nc")

echo "Running the sensitivity simulations..."
for (( i=0; i<${#pname[@]}; i++ ));
do
	for (( j=0; j<${#pfac[@]}; j++ ));
	do
		pvalue_new=`echo "${pvalue[i]}+${pvalue[i]}*${pfac[j]}" | bc -l`
		vi $filename -c ":%s/\("${pname[i]}" \{1,}= \).*\(,\)/\1"${pvalue_new}"\2/g" -c ":wq"
		nice -n 19 gotm &> log.resolute
		current_file_size=$(wc -c < "resolute.nc")
		while [[ $current_file_size != $required_file_size ]];
		do
			nice -n 19 gotm &> log.resolute
			current_file_size=$(wc -c < "resolute.nc")
		done
		if [[ "$j" -eq 0 ]];
		then
			pvalue_n[i]=`python sens_oat.py`
		elif [[ "$j" -eq 1 ]];
		then
			pvalue_p[i]=`python sens_oat.py`
		fi
	done
	neg=`echo "sqrt(${pvalue_n[i]}*${pvalue_n[i]})" | bc -l`
	pos=`echo "sqrt(${pvalue_p[i]}*${pvalue_p[i]})" | bc -l`
	pvalue_o[i]=`echo "${pos}+${neg}" | bc -l`
	vi $filename -c ":%s/\("${pname[i]}" \{1,}= \).*\(,\)/\1"${pvalue[i]}"\2/g" -c ":wq"
	echo "--->`echo "${i}+1"|bc -l` of ${#pname[@]} parameters done<---"
done
echo "${pname[@]}" > $results
echo "${pvalue[@]}" >> $results
echo "${pvalue_n[@]}" >> $results
echo "${pvalue_p[@]}" >> $results
echo "${pvalue_o[@]}" >> $results
echo "${pfac[@]}" >> $results
